@@ -1,13 +1,23 @@
 package Mojo::Content::Single;
 use Mojo::Base 'Mojo::Content';
 
-use Mojo::Asset::File;
 use Mojo::Asset::Memory;
 use Mojo::Content::MultiPart;
 
-has asset => sub { Mojo::Asset::Memory->new };
+has asset => sub { Mojo::Asset::Memory->new(auto_upgrade => 1) };
 has auto_upgrade => 1;
 
+sub new {
+  my $self = shift->SUPER::new(@_);
+  $self->{read} = $self->on(
+    read => sub {
+      my ($self, $chunk) = @_;
+      $self->asset($self->asset->add_chunk($chunk));
+    }
+  );
+  return $self;
+}
+
 sub body_contains {
   return 1 if shift->asset->contains(shift) >= 0;
   return;
@@ -39,42 +49,18 @@ sub get_body_chunk {
 sub parse {
   my $self = shift;
 
-  # Parse headers and chunked body
-  $self->SUPER::parse(@_);
-
-  # Still parsing headers or using a custom body parser
-  return $self
-    if ($self->{state} || '') eq 'headers' || $self->has_subscribers('read');
+  # Parse headers
+  $self->parse_until_body(@_);
 
   # Content needs to be upgraded to multipart
   if ($self->auto_upgrade && defined($self->boundary)) {
+    $self->unsubscribe(read => $self->{read});
     $self->emit(upgrade => my $multi = Mojo::Content::MultiPart->new($self));
     return $multi->parse;
   }
 
-  # Don't waste memory and upgrade to file based storage on demand
-  my $asset = $self->asset;
-  $self->asset($asset = Mojo::Asset::File->new->add_chunk($asset->slurp))
-    if $asset->isa('Mojo::Asset::Memory')
-      && $asset->size > ($ENV{MOJO_MAX_MEMORY_SIZE} || 262144);
-
-  # Chunked body or relaxed content
-  if ($self->is_chunked || $self->relaxed) {
-    $asset->add_chunk($self->{buffer});
-    $self->{buffer} = '';
-  }
-
-  # Normal body
-  else {
-    my $len = $self->headers->content_length || 0;
-    my $need = $len - $asset->size;
-    $asset->add_chunk(substr $self->{buffer}, 0, $need, '') if $need > 0;
-
-    # Finished
-    $self->{state} = 'finished' if $len <= $self->progress;
-  }
-
-  return $self;
+  # Parse body
+  return $self->SUPER::parse;
 }
 
 1;
@@ -142,6 +128,13 @@ Note that this attribute is EXPERIMENTAL and might change without warning!
 L<Mojo::Content::Single> inherits all methods from L<Mojo::Content> and
 implements the following new ones.
 
+=head2 C<new>
+
+  my $single = Mojo::Content::Single->new;
+
+Construct a new L<Mojo::Content::Single> object and register default C<read>
+event.
+
 =head2 C<body_contains>
 
   my $success = $single->body_contains('1234567');
@@ -169,9 +162,11 @@ Get a chunk of content starting from a specfic position.
 
 =head2 C<parse>
 
-  $single = $single->parse("Content-Length: 12\r\n\r\nHello World!");
+  $single   = $single->parse("Content-Length: 12\r\n\r\nHello World!");
+  my $multi = $single->parse("Content-Type: multipart/form-data\r\n\r\n");
 
-Parse content chunk.
+Parse content chunk and upgrade to L<Mojo::Content::MultiPart> object if
+possible.
 
 =head1 SEE ALSO
 
